package ALTLinux::RepoList::CLI;

use strict;
use warnings;
use Carp;
use Source::Shared::FindLocalMirror::ALTLinux;
use ALTLinux::RepoList qw(load_listfile_lambda_fn listdirs2srclists listdirs2binlists);
use Source::Shared::CLI;
our @ISA = qw/Exporter Source::Shared::CLI/;

our $VERSION=0.002;
#our $verbose=0;
#$verbose=1 if $ENV{TERM}; # not to pollute cron mail

our $need_bin=0;
our $need_src=0;
our @binlists;
our @srclists;

our $arch_allow_all=1;
our %arch_to_use;

require Exporter;
our @EXPORT_OK = qw/$arch_allow_all %arch_to_use/;


my @REPO;
my @listdir;
my ($opt_arch,$opt_arches);
my $infile;
my $opt_pipe;

our @LONGOPT=(
    'arch=s' => \$opt_arch,
    'arches=s' => \$opt_arches,
    'listdir=s' => \@listdir,
    'repo=s' => \@REPO,
    'bin-list=s' => \@binlists,
    'src-list=s' => \@srclists,
    'infile=s' => \$infile,
    'pipe|filter' => \$opt_pipe,
    );
sub print_longopt {
    print "  ALTLinux::RepoList options:
	--listdir <dir> /path/to/files/list/ dir
	--repo <dir> /path/to/repo dir (should have files/list subdir)
	--pipe	Pipe mode. Input is read from stdin and from files in cmdline args
		(normally, input is given as cmdline args or with --infile option).
	--infile <input file> read from <input file>. Not for --pipe mode.
";
    print "	--src-list /path/to/src.list
" if $need_src;
    print "	--bin-list /path/to/bin.list
	--arches noarch,x86_64,i586,armh the arches to use (default is all)
	--arch x86_64 same as --arches noarch,x86_64
" if $need_bin;

}

sub cli_container_ref {
    return ['Source::Shared::FindLocalMirror::ALTLinux'];
}

sub process_options {
    if ($need_bin and @binlists==0 or $need_src and @srclists==0) {
	push @listdir, map {-d $_.'/files/list' ? $_.'/files/list': $_} @REPO if @REPO;
	push @listdir, &Source::Shared::FindLocalMirror::ALTLinux::find_cli(-subpath=>'files/list') unless @listdir;
	push @srclists, &listdirs2srclists(@listdir);
	push @binlists, &listdirs2binlists(@listdir);
    }

    if ($need_bin and @binlists==0 or $need_src and @srclists==0) {
	warn "Can't find bin.list. Use --bin-list=/path\n" if $need_bin and @binlists==0;
	warn "Can't find src.list. Use --src-list=/path\n" if $need_src and @srclists==0;
	die "or use --listdir <dir> /path/to/files/list/ dir\n";
    }

    # process arches
    if ($opt_arches) {
	map {$arch_to_use{$_}=1}  split(',',$opt_arches);
	$arch_allow_all=0;
    };
    if ($opt_arch) {
	$arch_to_use{'noarch'}=1;
	$arch_to_use{$opt_arch}=1;
	$arch_allow_all=0;
    }
}

sub verify_args {
    if ($infile) {
	if ($opt_pipe) {
	    die "Usage error: --infile and --pipe can't be used together. Pass input files as command line arguments.";
	} elsif (scalar @ARGV) {
	    die "Usage error: mixed --infile and command line arguments.";
	}
    } elsif ($opt_pipe) {
	map {
	    -e $_ || die "Usage error: in --pipe mode command line arguments should be data files, but file ",$_," not found.";
	} @ARGV;
    }
}

sub compose_function_maps {
    my $func_array_ref = shift;
    my @in = @_;
    foreach my $func (@$func_array_ref) {
	@in = map { &$func($_) } @in;
    }
    return @in;
}

sub process_args {
    my ($self, $func_array_ref) = @_;
    if ($opt_pipe) {
	while (my $line=<>) {
	    chomp $line;
	    $line=~s/^\s*//;
	    next if $line=~m/^#/;
	    $line=~s/\s*$//;
	    print map {$_."\n"} map {&compose_function_maps($func_array_ref, $_)} $line;
	}
    } else {
	# ------- @IN --------------
	&load_listfile_lambda_fn(
	     $infile,
	     sub {
		 my ($fn)=@_;
		 while (my $line=<$fn>) {
		     chomp $line;
		     $line=~s/^\s*//;
		     next if $line=~m/^#/;
		     $line=~s/\s*$//;
		     print map {$_."\n"} map {&compose_function_maps($func_array_ref, $_)} $line;
		 }
	     }) if $infile;
	print map {$_."\n"} map {&compose_function_maps($func_array_ref, $_)} @ARGV;
	# ------- END @IN -----------
    }
}

1;
